﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using Microsoft.MediaCenter;
using Microsoft.MediaCenter.Hosting;
using Microsoft.MediaCenter.UI;

namespace webmc
{
	public class ForegroundApplication : ModelItem
	{
		private AddInHost host;
		private HistoryOrientedPageSession session;

		public ForegroundApplication()
			: this(null, null)
		{
		}

		public ForegroundApplication(HistoryOrientedPageSession session, AddInHost host)
		{
			this.session = session;
			this.host = host;
		}

		public MediaCenterEnvironment MediaCenterEnvironment
		{
			get
			{
				if (host == null)
				{
					return null;
				}
				return host.MediaCenterEnvironment;
			}
		}

		public void GoToMenu()
		{
			Dictionary<string, object> properties = new Dictionary<string, object>();
			properties["ForegroundApplication"] = this;

			if (session != null)
			{
				session.GoToPage("resx://webmc/webmc.Resources/Menu", properties);
			}
			else
			{
				Debug.WriteLine("GoToMenu");
			}
		}

		public void SoundSwitch(string deviceNumber)
		{
			String binaryPath;
			binaryPath = Path.Combine(Webmc.BinDir, "setaudio_itemn.exe");
			System.Diagnostics.Process process = new System.Diagnostics.Process();
			process.StartInfo.FileName = binaryPath;
			process.StartInfo.Arguments = deviceNumber.ToString();
			process.Start();
			process.WaitForExit();
		}

		public void DialogTest(string strClickedText)
		{
			int timeout = 5;
			bool modal = true;
			string caption = Resources.DialogCaption;

			if (session != null)
			{
				MediaCenterEnvironment.Dialog(
					strClickedText,
					caption,
					new object[] { DialogButtons.Ok },
					timeout,
					modal,
					null,
					delegate(DialogResult dialogResult) { }
				);
			}
			else
			{
				Debug.WriteLine("DialogTest");
			}
		}

	}
}