﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Albums : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/albums/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "[a-z]";
			}
		}

		private Dictionary<String, Int32> m_albumMediaAtoms = new Dictionary<String, Int32>();

		public Module_Albums()
		{
			WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
			String[] attributes;

			// Build a list of MediaAtoms for efficicency when looking up album info
			attributes = new[] {
					"Author",
					"AlbumID",
					"TrackingID",
					"WM/AlbumArtist",
					"WM/AlbumTitle",
					"WM/Genre",
					"WM/WMCollectionID",
					"WM/WMCollectionGroupID",
					"WM/WMContentID",
					"WM/Year"
				};
			foreach (String attribute in attributes)
			{
				m_albumMediaAtoms.Add(attribute, collection.getMediaAtom(attribute));
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				JArray jsonAlbums = new JArray();
				JObject jsonAlbum;

				WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
				WMPLib.IWMPQuery query;
				WMPLib.IWMPStringCollection albumIDs;
				WMPLib.IWMPPlaylist albumTracks;
				WMPLib.IWMPMedia track;

				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
#if DEBUF
						json.Add("x-param", param));
#endif
				query = collection.createQuery();
				if (param.Length > 0)
				{
					query.addCondition("WM/AlbumArtist", "BeginsWith", param);
				}

				//
				// Get a collection of album titles and loop over them
				//
				try
				{
					albumIDs = collection.getStringCollectionByQuery("AlbumID", query, "Audio", "Author", true);
				}
				catch (Exception ex)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, "getStringCollectionByQuery failed with query " + param + ": " + ex.ToString());
				}

				for (int albumIdIndex = 0; albumIdIndex < albumIDs.count; albumIdIndex++)
				{
					// Get a playlist for the album
					query = collection.createQuery();
					query.addCondition("AlbumID", "Equals", albumIDs.Item(albumIdIndex));
					albumTracks = collection.getPlaylistByQuery(query, "Audio", "WM/AlbumArtist", true);

					// Add a JSON object to hold the album, grab key details from the first track in it
					jsonAlbum = new JObject();
					track = albumTracks.get_Item(0);
					
					// Skip authors that match the letter but don't have an album
					if (!track.getItemInfoByAtom(m_albumMediaAtoms["WM/AlbumArtist"]).StartsWith(param, StringComparison.InvariantCultureIgnoreCase))
					{
						continue;
					}
					foreach (KeyValuePair<String, Int32> atom in m_albumMediaAtoms)
					{
						jsonAlbum.Add(atom.Key.ToString(), track.getItemInfoByAtom(atom.Value));
					}
					jsonAlbums.Add(jsonAlbum);
				}
				json.Add("albumsArray", jsonAlbums);
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}