﻿using System;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_DisplaySwitch : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/displayswitch/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "external|internal|clone|extend";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
				switch (param.ToLowerInvariant())
				{
					case "external":
					case "internal":
					case "clone":
					case "extend":
						break;

					default:
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
				String binaryPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.System), "DisplaySwitch.exe");
				json.Add("x-binaryPath", binaryPath);
				json.Add("x-param", param.ToLowerInvariant());
				System.Diagnostics.Process process = new System.Diagnostics.Process();
				process.StartInfo.FileName = binaryPath;
				process.StartInfo.Arguments = "/" + param.ToLowerInvariant();
				try
				{
					process.Start();
					process.WaitForExit();
					json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				}
				catch (Exception ex)
				{
					json.Add("x-exception", ex.ToString());
					json.Add(context.Request.Uri.AbsolutePath, false.ToString());
				}
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}
