﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using HttpServer;
using HttpServer.Headers;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Images : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/images/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "imagename";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				Bitmap bitmap;
				MemoryStream imageMs = new MemoryStream();
				String param;
				byte[] buffer;

				// Find the image in the resouces
				param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}

				// Handle modified checking, from FileModule.cs in HttpServer
				// We use the last modified time of the assembly as a base for client cache control
				DateHeader modifiedSince = context.Request.Headers["If-Modified-Since"] as DateHeader;
				if (modifiedSince != null)
				{
					if (modifiedSince.Value.CompareTo(Webmc.InstallDate) <= 0)
					{
						context.Response.Status = HttpStatusCode.NotModified;
					}
				}
				context.Response.Add(new DateHeader("Last-Modified", Webmc.InstallDate));

				if (context.Request.Method != Method.Header && context.Response.Status != HttpStatusCode.NotModified)
				{
					// Get the bitmap from the resource pool by is basename, and convert it to a PNG
					bitmap = (Bitmap)Resources.ResourceManager.GetObject(param);
					if (bitmap == null)
					{
						throw new HttpException(HttpStatusCode.BadRequest, "Error loading image: " + param);
					}
					bitmap.Save(imageMs, ImageFormat.Png);
					// Copy the image to the buffer
					int read;
					imageMs.Seek(0, SeekOrigin.Begin);
					buffer = new byte[imageMs.Length];
					read = imageMs.Read(buffer, 0, buffer.Length);
					if (read != buffer.Length)
					{
						throw new Exception("short read");
					}
				}
				else
				{
					buffer = new byte[] { 0 };
				}
				json.Add("image", buffer);

				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json, false);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}
