﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using HttpServer;
using HttpServer.Modules;
using Microsoft.MediaCenter;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_MediaMetaData : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/mediametadata";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH;
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}
			if (!MYPATH.EndsWith(Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath)))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				MediaMetadata metadata = WebmcBackgroundAddIn.Host.MediaMetadata;
				MediaTransport transport = WebmcBackgroundAddIn.Host.MediaTransport;
#if true
				if (WebmcBackgroundAddIn.Host.Current == null)
				{
					json.Add("x-no-WebmcBackgroundAddIn.Host.Current", "");
				}
				else
				{
					json.Add("x-yes-WebmcBackgroundAddIn.Host.Current", "");
					if (WebmcBackgroundAddIn.Host.MediaCenterEnvironment == null)
					{
						json.Add("x-no-WebmcBackgroundAddIn.Host.MediaCenterEnvironment", "");
					}
					else
					{
						json.Add("x-yes-WebmcBackgroundAddIn.Host.MediaCenterEnvironment", "");
						if (WebmcBackgroundAddIn.Host.MediaExperience == null)
						{
							json.Add("x-no-WebmcBackgroundAddIn.Host.MediaExperience-1", "");
							json.Add("x-Attempting _ResetWebmcMediaExperienceCache", "");
							WebmcBackgroundAddIn.Host._ResetWebmcMediaExperienceCache();
						}
						if (WebmcBackgroundAddIn.Host.MediaExperience == null)
						{
							json.Add("x-no-WebmcBackgroundAddIn.Host.MediaExperience-2", "");
						}
						else
						{
							json.Add("x-yes-WebmcBackgroundAddIn.Host.MediaExperience", "");
							if (WebmcBackgroundAddIn.Host.MediaMetadata == null)
							{
								json.Add("x-no-WebmcBackgroundAddIn.Host.MediaMetadata", "");
							}
							else
							{
								json.Add("x-yes-WebmcBackgroundAddIn.Host.MediaMetadata", "");
							}
						}
					}
				}
				if (Microsoft.MediaCenter.Hosting.AddInHost.Current == null)
				{
					json.Add("x-no-Microsoft.MediaCenter.Hosting.AddInHost.Current", "");
				}
				else
				{
					json.Add("x-yes-Microsoft.MediaCenter.Hosting.AddInHost.Current", "");
					if (Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment == null)
					{
						json.Add("x-no-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment", "");
					}
					else
					{
						json.Add("x-yes-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment", "");
						if (Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience == null)
						{
							json.Add("x-no-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience-1", "");
							json.Add("x-Attempting _ResetAddInHostMediaExperienceCache", "");
							WebmcBackgroundAddIn.Host._ResetAddInHostMediaExperienceCache();
						}
						if (Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience == null)
						{
							json.Add("x-no-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience-2", "");
						}
						else
						{
							json.Add("x-yes-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience", "");
							if (Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience.MediaMetadata == null)
							{
								json.Add("x-no-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience.MediaMetadata", "");
							}
							else
							{
								json.Add("x-yes-Microsoft.MediaCenter.Hosting.AddInHost.Current.MediaCenterEnvironment.MediaExperience.MediaMetadata", "");
							}
						}
					}
				}
#endif
				if (metadata == null)
				{
					// not an error, happens when we are not playing anything
					json.Add("x-no-MediaMetadata", "");
					json.Add("PlayState", Microsoft.MediaCenter.PlayState.Undefined.ToString());
				}
				else if (transport == null)
				{
					// not an error, happens when we are not playing anything
					json.Add("x-no-MediaTransport", "");
					json.Add("PlayState", Microsoft.MediaCenter.PlayState.Undefined.ToString());
				}
				else
				{
					json.Add("PlayState", transport.PlayState.ToString());
					json.Add("Buffering", transport.BufferingProgress.ToString());
					json.Add("PlayRate", transport.PlayRate.ToString());
					json.Add("Position", transport.Position.ToString());
					foreach (KeyValuePair<String, object> entry in metadata)
					{
						if (entry.Key.ToString() == "TrackDuration")
						{
							Double duration = Convert.ToDouble(entry.Value.ToString());
							Double h = Math.Floor(duration / 3600);
							Double m = Math.Floor((duration - h * 3600) / 60);
							Double s = Math.Floor((duration - h * 3600 - m * 60));
							StringBuilder sb = new StringBuilder();
							sb.AppendFormat("{0:D2}:{1:D2}:{2:D2}", Convert.ToInt32(h), Convert.ToInt32(m), Convert.ToInt32(s));
							json.Add(entry.Key.ToString(), sb.ToString());
						}
						else
						{
							json.Add(entry.Key.ToString(), entry.Value.ToString());
						}
					}
				}
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}