﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Diagnostics;
using System.Reflection;
using System.Text;
using HttpServer;
using HttpServer.Headers;
using HttpServer.Modules;

namespace webmc
{
	class Module_MissingPage : IModule
	{
		private IModule[] m_modules;

		public Module_MissingPage(IModule[] modules)
		{
			m_modules = modules;
		}

		public ProcessingResult Process(RequestContext context)
		{
			try
			{
				StringBuilder sb = new StringBuilder();
				sb.AppendLine("<html>");
				sb.AppendLine("<body>");
				sb.AppendLine("Missing Page: " + context.Request.Uri.AbsolutePath);
				sb.AppendLine("<hr>");
				for (var moduleIndex = 0; moduleIndex < m_modules.Length; moduleIndex++)
				{
					IWebmcModule module = (IWebmcModule)m_modules[moduleIndex];
					sb.AppendFormat("{0}<br>", module.USAGE);
				}
				sb.AppendLine("<hr>");
				FileVersionInfo vi = FileVersionInfo.GetVersionInfo(Assembly.GetExecutingAssembly().Location);
				foreach (PropertyInfo p in vi.GetType().GetProperties())
				{
					// Write the name and the value.
					sb.AppendFormat("{0} = {1}<br>", p.Name, p.GetValue(vi, null));
				}
#if false
				AssemblyName an = Assembly.GetExecutingAssembly().GetName();
				sb.AppendLine("<hr>");
				foreach (PropertyInfo p in an.GetType().GetProperties())
				{
					// Write the name and the value.
					sb.AppendFormat("{0} = {1}<br>", p.Name, p.GetValue(vi, null));
				}
#endif
				sb.AppendLine("<hr>");
				sb.AppendFormat("{0}<br>", Webmc.ProgramFilesx86);
				sb.AppendFormat("{0}<br>", Webmc.HtDocsDir);
				sb.AppendFormat("{0}<br>", Webmc.InstallDate.ToLongDateString());
				sb.AppendFormat("{0}<br>", Webmc.BinDir);
				sb.AppendLine("</body>");
				sb.AppendLine("</html>");

				byte[] buffer = Encoding.UTF8.GetBytes(sb.ToString());

				context.Response.Status = System.Net.HttpStatusCode.BadRequest;
				context.Response.ContentType.Value = "text/html";
				context.Response.Add(new StringHeader("Cache-Control", "no-cache, must-revalidate"));
				context.Response.Add(new StringHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT"));
				context.Response.ContentLength.Value = buffer.Length;
				context.Response.Body.Write(buffer, 0, buffer.Length);

				return ProcessingResult.SendResponse;
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}