﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Microsoft.MediaCenter;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Play : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/play/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "WMContentID|WMCollectionGroupID|TrackingID";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}
			return PlayQueue.Process(context, false, MYPATH, USAGE);
		}
	}

	class Module_Queue : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/queue/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "WMContentID|WMCollectionGroupID|TrackingID";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}
			return PlayQueue.Process(context, true, MYPATH, USAGE);
		}
	}

	internal class PlayQueue
	{
		public static ProcessingResult Process(RequestContext context, Boolean queue, String MYPATH, String USAGE)
		{
			try
			{
				JObject json = new JObject();

				MediaCenterEnvironment environment = WebmcBackgroundAddIn.Host.MediaCenterEnvironment;

				WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
				WMPLib.IWMPQuery query;
				WMPLib.IWMPStringCollection sourceURL = null;

				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
#if DEBUG
				json.Add("x-param", param);
				json.Add("x-param-unescaped", Uri.UnescapeDataString(param));
#endif
				if (environment == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No MediaCenterEnvironment");
				}
				else if (collection == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No mediaCollection");
				}
				else
				{
					//
					// Look up the track by its WMContentID, WMCollectionGroupID, TrackingID to find the sourceURL
					//
					String[][] conditions = new[] { 
						new[] {"WM/WMContentID", "Audio",},
						new[] {"WM/WMCollectionGroupID", "Audio",},
						new[] {"TrackingID", "Playlist"}
					};
					foreach (String[] condition in conditions)
					{
						query = collection.createQuery();
						query.addCondition(condition[0], "Equals", Uri.UnescapeDataString(param));
						try
						{
							sourceURL = collection.getStringCollectionByQuery("SourceURL", query, condition[1], "WM/TrackNumber", true);
						}
						catch (Exception ex)
						{
							throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Query failed using GUID " + Uri.UnescapeDataString(param) + "<br />" + ex.ToString());
						}
						if (sourceURL.count != 0)
						{
							break;
						}
					}
					if (sourceURL == null || sourceURL.count == 0)
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, "No item with GUID " + Uri.UnescapeDataString(param));
					}

					for (var trackIndex = 0; trackIndex < sourceURL.count; trackIndex++)
					{
						if (!queue)
						{
							queue = (trackIndex != 0);
						}
						environment.PlayMedia(MediaType.Audio, sourceURL.Item(trackIndex), queue);
					}
				}
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}