﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Microsoft.MediaCenter;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_PlayRate : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/playrate/";
			}
		}
		private String _USAGE = null;
		public String USAGE
		{
			get
			{
				if (_USAGE == null)
				{
					return MYPATH;
				}
				else
				{
					return _USAGE;
				}
			}
			set
			{
				_USAGE = value;
			}
		}

		public enum PlayRates {
			stop,
			pause,
			play,
			ff1,
			ff2,
			FF3,
			rewind1,
			rewind2,
			rewind3,
			slowmotion1,
			slowmotion2,
			slowmotion3
		};

		public Module_PlayRate() {
			foreach (String s in Enum.GetNames(typeof(PlayRates))) {
				USAGE = USAGE + s + "|";
			}
			USAGE = USAGE.TrimEnd(new[] { '|' });
			USAGE = USAGE;
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				MediaTransport transport = WebmcBackgroundAddIn.Host.MediaTransport;
				if (transport == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No MediaTransport");
				}
				else
				{
					String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
					if (param.Length == 0)
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
					} 
					if (Enum.IsDefined(typeof(PlayRates), param.ToLowerInvariant()))
					{
						transport.PlayRate = Convert.ToInt32(Enum.Parse(typeof(PlayRates), param.ToLowerInvariant()));
					}
					else
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
					}
				}
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}