﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_SoundSwitch : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/soundswitch/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "1|2|...";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
				else
				{
					int deviceNumber;
					try
					{
						deviceNumber = int.Parse(param);
					}
					catch
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
					}
					if (deviceNumber > 50 || deviceNumber < 0)
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
					}

					String binaryPath;
					binaryPath = Path.Combine(Webmc.BinDir, "setaudio_itemn.exe");
					json.Add("x-binaryPath", binaryPath);
					json.Add("x-deviceNumber", deviceNumber.ToString());
					System.Diagnostics.Process process = new System.Diagnostics.Process();
					process.StartInfo.FileName = binaryPath;
					process.StartInfo.Arguments = deviceNumber.ToString();
					try
					{
						process.Start();
						process.WaitForExit();
						json.Add(context.Request.Uri.AbsolutePath, true.ToString());
					}
					catch (Exception ex)
					{
						json.Add("x-exception", ex.ToString());
						json.Add(context.Request.Uri.AbsolutePath, false.ToString());
					}
					return OutputData.Output(context, json);
				}
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}

	}
}
