﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_UnknownAlbums : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/unknownalbums";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH;
			}
		}

		private Dictionary<String, Int32> m_trackMediaAtoms = new Dictionary<String, Int32>();

		public Module_UnknownAlbums()
		{
			WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
			String[] attributes;

			// Build a list of MediaAtoms for efficicency when looking up album info
			attributes = new[] {
					"Author",
					"AlbumID",
					"Title",
					"TrackingID",
					"WM/AlbumArtist",
					"WM/AlbumTitle",
					"WM/Genre",
					"WM/WMCollectionID",
					"WM/WMCollectionGroupID",
					"WM/WMContentID",
					"WM/Year"
				};
			foreach (String attribute in attributes)
			{
				m_trackMediaAtoms.Add(attribute, collection.getMediaAtom(attribute));
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				JArray jsonTracks = new JArray();
				JObject jsonTrack;

				WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
				WMPLib.IWMPQuery query;
				WMPLib.IWMPPlaylist tracks;
				WMPLib.IWMPMedia track;

				if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
				{
					return ProcessingResult.Continue;
				}
				if (!MYPATH.EndsWith(Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath)))
				{
					return ProcessingResult.Continue;
				}

				//
				// Get a playlist of tracks that have am empty AlbumID
				//
				query = collection.createQuery();
				query.addCondition("AlbumID", "Equals", "");
				tracks = collection.getPlaylistByQuery(query, "Audio", "Author", true);

				for (int trackIndex = 0; trackIndex < tracks.count; trackIndex++)
				{
					track = tracks.get_Item(trackIndex);
					jsonTrack = new JObject();
#if DEBUG
					String attrName, attrValue;
					for (int trackAttrIndex = 0; trackAttrIndex < track.attributeCount; trackAttrIndex++)
					{
						attrName = track.getAttributeName(trackAttrIndex);
						attrValue = track.getItemInfo(attrName);
						jsonTrack.Add("Looped-" + attrName.ToString(), attrValue.ToString());
					}
#endif
					foreach (KeyValuePair<String, Int32> atom in m_trackMediaAtoms)
					{
						jsonTrack.Add(atom.Key.ToString(), track.getItemInfoByAtom(atom.Value));
					}
					jsonTracks.Add(jsonTrack);
				}
				json.Add("tracksArray", jsonTracks);
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}