﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Microsoft.MediaCenter;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Volume : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/volume/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "Up|Down|Mute|Unmute|(0:50)";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				AudioMixer mixer = WebmcBackgroundAddIn.Host.AudioMixer;

				if (mixer == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No AudioMixer");
				}
				else
				{
					String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
					if (param.Length == 0)
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
					}
					if (param.Equals("Up", StringComparison.InvariantCultureIgnoreCase))
					{
						mixer.VolumeUp();
					}
					else if (param.Equals("Down", StringComparison.InvariantCultureIgnoreCase))
					{
						mixer.VolumeDown();
					}
					else if (param.Equals("Mute", StringComparison.InvariantCultureIgnoreCase))
					{
						mixer.Mute = true;
					}
					else if (param.Equals("UnMute", StringComparison.InvariantCultureIgnoreCase))
					{
						mixer.Mute = false;
					}
					else
					{
						int desiredLevel;
						try
						{
							desiredLevel = int.Parse(param);
						}
						catch
						{
							throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
						}
						if (desiredLevel > 50 || desiredLevel < 0)
						{
							throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
						}
						int volume = (int)(mixer.Volume / 1310.7);
						for (int level = volume; level > desiredLevel; level--)
						{
							mixer.VolumeDown();
						}

						for (int level = volume; level < desiredLevel; level++)
						{
							mixer.VolumeUp();
						}
					}
					json.Add("volumeLevel", mixer.Volume.ToString());
				}
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}