﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.IO;
using System.Text;
using HttpServer;
using HttpServer.Headers;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class OutputData
	{
		public static ProcessingResult Output(RequestContext context, JToken json)
		{
			return Output(context, json, true);
		}
		public static ProcessingResult Output(RequestContext context, JToken json, Boolean noCache)
		{
			byte[] buffer;
			StringBuilder sb = new StringBuilder();

			switch (Path.GetExtension(context.Request.Uri.AbsolutePath).TrimStart('.'))
			{
				case "json":
					buffer = Encoding.UTF8.GetBytes(json.ToString());
					context.Response.ContentType.Value = "application/json";
					break;

				case "png":
					buffer = json.Value<byte[]>("image");
					context.Response.ContentType.Value = "image/png";
					break;

				case "html":
				default:
					sb.AppendLine("<html>");
					sb.AppendLine("<body>");
					sb.AppendLine("<pre>");
					sb.AppendLine(json.ToString());
					sb.AppendLine("</pre>");
					sb.AppendLine("</body>");
					sb.AppendLine("</html>");

					buffer = Encoding.UTF8.GetBytes(sb.ToString());
					context.Response.ContentType.Value = "text/html";
					break;
			}
			if (noCache)
			{
				context.Response.Add(new StringHeader("Cache-Control", "no-cache, must-revalidate"));
				context.Response.Add(new StringHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT"));
			}
			context.Response.ContentLength.Value = buffer.Length;
			context.Response.Body.Write(buffer, 0, buffer.Length);

			return ProcessingResult.SendResponse;
		}
	}
}
