﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;

namespace webmc
{
	class Webmc
	{
		public static DateTime InstallDate
		{
			get
			{
				return File.GetLastWriteTimeUtc(Assembly.GetExecutingAssembly().Location);
			}
		}

		public static string ProgramFilesx86
		{
			get
			{
				if (8 == IntPtr.Size || (!String.IsNullOrEmpty(Environment.GetEnvironmentVariable("PROCESSOR_ARCHITEW6432"))))
				{
					return Environment.GetEnvironmentVariable("ProgramFiles(x86)");
				}
				return Environment.GetEnvironmentVariable("ProgramFiles");
			}
		}

		public static string HtDocsDir
		{
			get
			{
				FileVersionInfo vi = FileVersionInfo.GetVersionInfo(Assembly.GetExecutingAssembly().Location);
				return Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) +
					Path.DirectorySeparatorChar + vi.CompanyName +
					Path.DirectorySeparatorChar + vi.ProductName +
					Path.DirectorySeparatorChar + "htdocs";
			}
		}

		public static string BinDir
		{
			get
			{
				FileVersionInfo vi = FileVersionInfo.GetVersionInfo(Assembly.GetExecutingAssembly().Location);
				String binPath;
				binPath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) +
					Path.DirectorySeparatorChar + vi.CompanyName +
					Path.DirectorySeparatorChar + vi.ProductName +
					Path.DirectorySeparatorChar + "bin";
				if (Directory.Exists(binPath))
				{
					return binPath;
				}
				binPath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + " (x86)" +
					Path.DirectorySeparatorChar + vi.CompanyName +
					Path.DirectorySeparatorChar + vi.ProductName +
					Path.DirectorySeparatorChar + "bin";
				if (Directory.Exists(binPath))
				{
					return binPath;
				}
				return String.Empty;
			}
		}

		public static void Log(string msg)
		{
			StreamWriter sw = File.AppendText("c:\\webmclog.txt");
			sw.WriteLine(msg);
			sw.Close();
		}

	}
}
